import random
import string
import datetime

from django import forms
from django.conf import settings
from django.template.defaultfilters import slugify
from django.utils.translation import ugettext_lazy as _

from common.forms import RenderableMixin
from marker.models import Marker, Type, Tag


class SplitLifetimeWidget(forms.MultiWidget):
    def __init__(self, attrs=None, date_format=None, time_format=None):
        widgets = (forms.TextInput(attrs=attrs),
                   forms.Select(attrs=attrs, 
                                choices=(
                                    ('d', _('days')),
                                    ('w', _('weeks')), 
                                    ('y', _('years')))))
        super(SplitLifetimeWidget, self).__init__(widgets, attrs)

    def decompress(self, value):
        if value and len(value) >= 2:
            return [value[0:-2], value[-1]]
        return [None, None]

class AddMarkerForm(forms.ModelForm, RenderableMixin):
    lifetime = forms.CharField(label=_('Lifetime'), 
                               required=False, 
                               initial='1y',
                               widget=SplitLifetimeWidget(attrs={'class': 'small'}), 
                               help_text=_('How long should the marker be visible on the map'))
    
    tags = forms.CharField(label=_('Tags (comma separated)'), required=False)
    
    class Meta:
        model = Marker
        fields = ('title', 'more_info_url', 'lat', 'lng')
        
    def __init__(self, marker_type, *args, **kwargs):
        super(AddMarkerForm, self).__init__(*args, **kwargs)
        self.type_object = Type.objects.get(pk=marker_type)
        self.content_slug_fields = []
        self.fields.keyOrder = ['title']
        for attr in self.type_object.allowed_attibutes.all():
            self.content_slug_fields.append(slugify(attr.attribute))
            self.fields[slugify(attr.attribute)] = forms.CharField(label=attr.attribute, required=False)
        self.fields.keyOrder.extend(['more_info_url', 'lat', 'lng'])
    
    def clean_lifetime(self):
        if len(self.cleaned_data.get('lifetime', '')) < 12: #len([u'', u'y']) == 11
            raise forms.ValidationError(_('Lifetime is required'))
        if self.get_timedelta() is None:
            raise forms.ValidationError(_('Lifetime has exceeded the maximum lifetime'))
        return self.cleaned_data.get('lifetime', '')
        
    def get_timedelta(self):
        timedelta = None
        
        try:
            lifetime = int(self.cleaned_data.get('lifetime')[3:-2].split('\', u\'')[0])
            lifetime_type = self.cleaned_data.get('lifetime')[3:-2].split('\', u\'')[1]
        except:
            pass
        else:
            if lifetime_type == 'd':
                timedelta = datetime.timedelta(days=lifetime)
            elif lifetime_type == 'w':
                timedelta = datetime.timedelta(weeks=lifetime)
            elif lifetime_type == 'y':
                timedelta = datetime.timedelta(weeks=lifetime*52)

        if timedelta is None:
            timedelta = getattr(settings, 'DEFAULT_MARKER_LIFETIME')

        if timedelta > getattr(settings, 'MAX_MARKER_LIFETIME'):
            return None
        else:
            return timedelta
    
    def save(self, *args, **kwargs):
        self.instance.secret = ''.join(random.sample(string.letters+string.digits, 12))
        self.instance.expiration_date = datetime.date.today()
        self.instance.marker_type = self.type_object
        super(AddMarkerForm, self).save(*args, **kwargs)
        if self.cleaned_data.get('tags'):
            for t in self.cleaned_data.get('tags').split(','):
                if t.strip() is not '':
                    tag, created = Tag.objects.get_or_create(tag=t.strip())
                    self.instance.tags.add(tag)
        content_dict = {}
        for item in self.cleaned_data.iteritems():
            if item[0] in self.content_slug_fields:
                content_dict[item[0]] = item[1]
        self.instance.set_content(content_dict)
        self.instance.expiration_date = self.instance.update_date + self.get_timedelta()
        self.instance.save()
        return self.instance
        

class UpdateMarkerForm(AddMarkerForm):
    def save(self, *args, **kwargs):
        self.instance.expiration_date = datetime.date.today()
        super(AddMarkerForm, self).save(*args, **kwargs)
        for t in self.cleaned_data.get('tags').split(','):
            if t.strip() is not '':
                tag, created = Tag.objects.get_or_create(tag=t.strip())
                self.instance.tags.add(tag)
        self.instance.expiration_date = self.instance.update_date + self.get_timedelta()
        self.instance.save()
        return self.instance
            
