import datetime
import logging
import pickle
import base64

from django.db import models
from django.utils.translation import ugettext_lazy as _

logger = logging.getLogger('mainlogger')


class Tag(models.Model):
    tag = models.CharField(max_length=25)
    
    def __unicode__(self):
        return self.tag


class TypeAttribute(models.Model):
    attribute = models.CharField(max_length=50)
    
    def __unicode__(self):
        return self.attribute
        

class Type(models.Model):
    name = models.CharField(max_length=25)
    description = models.TextField()
    active = models.BooleanField(default=True)
    marker_icon = models.ImageField(upload_to='markers')
    template = models.FileField(upload_to='templates') # template and content dict should match and combined to HTML output
    allowed_attibutes = models.ManyToManyField(TypeAttribute)
    default_lifetime = models.DateField(default=datetime.timedelta(weeks=52))
    max_lifetime = models.DateField(default=datetime.timedelta(weeks=3*52))
    
    def __unicode__(self):
        return self.name
        
    def json_dict(self):
        return {'name': self.name,
                'description': self.description,
                'marker_icon': self.marker_icon.url,
                'template': self.template.url}


class Marker(models.Model):
    lat = models.FloatField()
    lng = models.FloatField()
    marker_type = models.ForeignKey(Type)
    tags = models.ManyToManyField(Tag)
    title = models.CharField(max_length=50, verbose_name=_('Title'))
    content = models.TextField(verbose_name=_('Content')) # json dictionary
    more_info_url = models.URLField(null=True, blank=True, verbose_name=_('Info URL'))
    secret = models.CharField(max_length=16)
    creation_date = models.DateField(auto_now_add=True)
    update_date = models.DateField(auto_now=True)
    expiration_date = models.DateField(blank=False)
    
    def __unicode__(self):
        return '%s [%s, %s]' % (self.title, self.lat, self.lng)
        
    def set_content(self, content_dict):
        self.content = base64.b64encode(pickle.dumps(content_dict))
        
    def get_content(self):
        try:
            return pickle.loads(base64.b64decode(self.content))
        except:
            logging.error('marker content with id=%d cannot be unpickled' % self.pk)
            return {}
        
    def json_dict(self):
        return {'lat': self.lat,
                'lng': self.lng,
                'marker_type': self.marker_type.json_dict(),
                'tags': [tag.tag for tag in self.tags.all()],
                'title': self.title,
                'content': self.get_content(),
                'more_info_url': self.more_info_url,
                'icon': self.marker_type.marker_icon.url,
                'type': self.marker_type.name}
