import urllib
import logging
import datetime
import GeoIP
import json

from django.http import HttpResponse, QueryDict, HttpResponseRedirect
from django.core import serializers
from django.core.urlresolvers import reverse
from django.forms.models import model_to_dict
from django.views.generic.simple import direct_to_template

from marker.forms import AddMarkerForm, UpdateMarkerForm
from marker.models import Marker, Type, Tag
from marker.encoders import MarkerEncoder

logger = logging.getLogger('mainlogger')

def index(request):
    marker_types = Type.objects.filter(active=True)
    return direct_to_template(request, 'index.html', extra_context={'types': marker_types})
    
def add_marker(request, marker_type):
    form = AddMarkerForm(data=request.POST or None, marker_type=marker_type)
    
    if request.POST and form.is_valid():
        instance = form.save()
        return HttpResponseRedirect(reverse('map'))
    return direct_to_template(request, 'create_marker.html', extra_context={'form': form})

def show_map(request):
    return direct_to_template(request, 'full_map.html')
    
def all_markers(request):
    # Since django 1.1 does not support natural keys, do this the manual way
    # http://docs.djangoproject.com/en/dev/topics/serialization/#natural-keys
    markers = MarkerEncoder(ensure_ascii=False).encode(Marker.objects.filter(expiration_date__gt=datetime.date.today()))
    return HttpResponse(markers, mimetype='application/json')
    
    
# API
def register(request):
    form = AddMarkerForm(request.POST or None)
    
    if request.POST and form.is_valid():
        form.save()
        secret_response = MarkerEncoder(ensure_ascii=False).encode({'secret': form.instance.secret})
        return HttpResponse(secret_response, mimetype='application/json')
        
    errors = {'errors': [{a[0]: '%'.join(a[1])} for a in form.errors.iteritems()]}
    error_response = MarkerEncoder(ensure_ascii=False).encode(errors)
    return HttpResponse(error_response, mimetype='application/json')

def unregister(request, secret):
    success = False
    response_dict = {}
    try:
        marker = Marker.objects.get(secret=secret)
    except Marker.DoesNotExist:
        response_dict.update({'errors': 'secret is not right'})
    else:
        marker.delete()
        success = True
    
    response_dict.update({'success': success})
    return HttpResponse(MarkerEncoder(ensure_ascii=False).encode(response_dict), mimetype='application/json')

def update(request, secret):
    success = False
    response_dict = {}
    try:
        marker = Marker.objects.get(secret=secret)
    except Marker.DoesNotExist:
        response_dict.update({'errors': 'secret is not right'})
    else:
        post_data = request.POST.copy()
        model_data = model_to_dict(marker, fields=['lat', 'lng', 'marker_type', 'tags', 'title', 'content', 'more_info_url'])
        for key, value in model_data.iteritems():
            if key == 'tags':
                value = ','.join([Tag.objects.get(pk=tag_id).tag for tag_id in value])
            post_data.setdefault(key, value)
        form = UpdateMarkerForm(post_data, instance=marker)
        
        if post_data and form.is_valid():
            form.save()
            success = True
        else:    
            response_dict.update({'errors': [{a[0]: '%'.join(a[1])} for a in form.errors.iteritems()]})
        
    response_dict.update({'success': success})
    return HttpResponse(MarkerEncoder(ensure_ascii=False).encode(response_dict), mimetype='application/json')

def geoip(request):
    clientIP = request.GET.get('ip')
    if not clientIP:
        clientIP = request.META['REMOTE_ADDR']
        if clientIP == '127.0.0.1': # For developers
            clientIP = urllib.urlopen('http://www.biranchi.com/ip.php').read()
    gi = GeoIP.open('/usr/share/GeoIP/GeoLiteCity.dat', GeoIP.GEOIP_STANDARD)
    gir = gi.record_by_addr(clientIP)
    logger.debug( 'Request from %s, gir: %s' % (clientIP, gir))
    return HttpResponse(json.dumps(gir), mimetype='application/json')

def types(request):
    data = serializers.serialize('json', Type.objects.filter(active=True), fields=('name','description'))
    return HttpResponse(data, mimetype='application/json')
